/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.4.0 (build: 2013-06-26 16:27:34 UTC)
 *  on 2013-07-30 at 19:28:56 UTC 
 */

package com.google.api.services.shopping;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Shopping (v1).
 *
 * <p>
 * Lets you search over product data.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/shopping-search/v1/getting_started" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link ShoppingRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Shopping extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the Search API For Shopping library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "shopping/search/v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Shopping(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Shopping(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Products collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Shopping shopping = new Shopping(...);}
   *   {@code Shopping.Products.List request = shopping.products().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Products products() {
    return new Products();
  }

  /**
   * The "products" collection of methods.
   */
  public class Products {

    /**
     * Returns a single product
     *
     * Create a request for the method "products.get".
     *
     * This request holds the parameters needed by the the shopping server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param source Query source
     * @param accountId Merchant center account id
     * @param productIdType Type of productId
     * @param productId Id of product
     * @return the request
     */
    public Get get(String source, Long accountId, String productIdType, String productId) throws java.io.IOException {
      Get result = new Get(source, accountId, productIdType, productId);
      initialize(result);
      return result;
    }

    public class Get extends ShoppingRequest<com.google.api.services.shopping.model.Product> {

      private static final String REST_PATH = "{source}/products/{accountId}/{productIdType}/{productId}";

      /**
       * Returns a single product
       *
       * Create a request for the method "products.get".
       *
       * This request holds the parameters needed by the the shopping server.  After setting any
       * optional parameters, call the {@link Get#execute()} method to invoke the remote operation. <p>
       * {@link Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param source Query source
       * @param accountId Merchant center account id
       * @param productIdType Type of productId
       * @param productId Id of product
       * @since 1.13
       */
      protected Get(String source, Long accountId, String productIdType, String productId) {
        super(Shopping.this, "GET", REST_PATH, null, com.google.api.services.shopping.model.Product.class);
        this.source = Preconditions.checkNotNull(source, "Required parameter source must be specified.");
        this.accountId = Preconditions.checkNotNull(accountId, "Required parameter accountId must be specified.");
        this.productIdType = Preconditions.checkNotNull(productIdType, "Required parameter productIdType must be specified.");
        this.productId = Preconditions.checkNotNull(productId, "Required parameter productId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Query source */
      @com.google.api.client.util.Key
      private String source;

      /** Query source
       */
      public String getSource() {
        return source;
      }

      /** Query source */
      public Get setSource(String source) {
        this.source = source;
        return this;
      }

      /** Merchant center account id */
      @com.google.api.client.util.Key
      private Long accountId;

      /** Merchant center account id
       */
      public Long getAccountId() {
        return accountId;
      }

      /** Merchant center account id */
      public Get setAccountId(Long accountId) {
        this.accountId = accountId;
        return this;
      }

      /** Type of productId */
      @com.google.api.client.util.Key
      private String productIdType;

      /** Type of productId
       */
      public String getProductIdType() {
        return productIdType;
      }

      /** Type of productId */
      public Get setProductIdType(String productIdType) {
        this.productIdType = productIdType;
        return this;
      }

      /** Id of product */
      @com.google.api.client.util.Key
      private String productId;

      /** Id of product
       */
      public String getProductId() {
        return productId;
      }

      /** Id of product */
      public Get setProductId(String productId) {
        this.productId = productId;
        return this;
      }

      /** Category specification */
      @com.google.api.client.util.Key("categories.include")
      private String categoriesInclude;

      /** Category specification
       */
      public String getCategoriesInclude() {
        return categoriesInclude;
      }

      /** Category specification */
      public Get setCategoriesInclude(String categoriesInclude) {
        this.categoriesInclude = categoriesInclude;
        return this;
      }

      /** Whether to return recommendation information */
      @com.google.api.client.util.Key("recommendations.enabled")
      private Boolean recommendationsEnabled;

      /** Whether to return recommendation information
       */
      public Boolean getRecommendationsEnabled() {
        return recommendationsEnabled;
      }

      /** Whether to return recommendation information */
      public Get setRecommendationsEnabled(Boolean recommendationsEnabled) {
        this.recommendationsEnabled = recommendationsEnabled;
        return this;
      }

      /** Thumbnail specification */
      @com.google.api.client.util.Key
      private String thumbnails;

      /** Thumbnail specification
       */
      public String getThumbnails() {
        return thumbnails;
      }

      /** Thumbnail specification */
      public Get setThumbnails(String thumbnails) {
        this.thumbnails = thumbnails;
        return this;
      }

      /** Merchant taxonomy */
      @com.google.api.client.util.Key
      private String taxonomy;

      /** Merchant taxonomy
       */
      public String getTaxonomy() {
        return taxonomy;
      }

      /** Merchant taxonomy */
      public Get setTaxonomy(String taxonomy) {
        this.taxonomy = taxonomy;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("categories.useGcsConfig")
      private Boolean categoriesUseGcsConfig;

      /** This parameter is currently ignored
       */
      public Boolean getCategoriesUseGcsConfig() {
        return categoriesUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public Get setCategoriesUseGcsConfig(Boolean categoriesUseGcsConfig) {
        this.categoriesUseGcsConfig = categoriesUseGcsConfig;
        return this;
      }

      /** Recommendation specification */
      @com.google.api.client.util.Key("recommendations.include")
      private String recommendationsInclude;

      /** Recommendation specification
       */
      public String getRecommendationsInclude() {
        return recommendationsInclude;
      }

      /** Recommendation specification */
      public Get setRecommendationsInclude(String recommendationsInclude) {
        this.recommendationsInclude = recommendationsInclude;
        return this;
      }

      /** Whether to return category information */
      @com.google.api.client.util.Key("categories.enabled")
      private Boolean categoriesEnabled;

      /** Whether to return category information
       */
      public Boolean getCategoriesEnabled() {
        return categoriesEnabled;
      }

      /** Whether to return category information */
      public Get setCategoriesEnabled(Boolean categoriesEnabled) {
        this.categoriesEnabled = categoriesEnabled;
        return this;
      }

      /** Location used to determine tax and shipping */
      @com.google.api.client.util.Key
      private String location;

      /** Location used to determine tax and shipping
       */
      public String getLocation() {
        return location;
      }

      /** Location used to determine tax and shipping */
      public Get setLocation(String location) {
        this.location = location;
        return this;
      }

      /** Comma separated list of attributes to return */
      @com.google.api.client.util.Key
      private String attributeFilter;

      /** Comma separated list of attributes to return
       */
      public String getAttributeFilter() {
        return attributeFilter;
      }

      /** Comma separated list of attributes to return */
      public Get setAttributeFilter(String attributeFilter) {
        this.attributeFilter = attributeFilter;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("recommendations.useGcsConfig")
      private Boolean recommendationsUseGcsConfig;

      /** This parameter is currently ignored
       */
      public Boolean getRecommendationsUseGcsConfig() {
        return recommendationsUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public Get setRecommendationsUseGcsConfig(Boolean recommendationsUseGcsConfig) {
        this.recommendationsUseGcsConfig = recommendationsUseGcsConfig;
        return this;
      }

    }
    /**
     * Returns a list of products and content modules
     *
     * Create a request for the method "products.list".
     *
     * This request holds the parameters needed by the the shopping server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param source Query source
     * @return the request
     */
    public List list(String source) throws java.io.IOException {
      List result = new List(source);
      initialize(result);
      return result;
    }

    public class List extends ShoppingRequest<com.google.api.services.shopping.model.Products> {

      private static final String REST_PATH = "{source}/products";

      /**
       * Returns a list of products and content modules
       *
       * Create a request for the method "products.list".
       *
       * This request holds the parameters needed by the the shopping server.  After setting any
       * optional parameters, call the {@link List#execute()} method to invoke the remote operation. <p>
       * {@link List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param source Query source
       * @since 1.13
       */
      protected List(String source) {
        super(Shopping.this, "GET", REST_PATH, null, com.google.api.services.shopping.model.Products.class);
        this.source = Preconditions.checkNotNull(source, "Required parameter source must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Query source */
      @com.google.api.client.util.Key
      private String source;

      /** Query source
       */
      public String getSource() {
        return source;
      }

      /** Query source */
      public List setSource(String source) {
        this.source = source;
        return this;
      }

      /** Facets to include (applies when useGcsConfig == false) */
      @com.google.api.client.util.Key("facets.include")
      private String facetsInclude;

      /** Facets to include (applies when useGcsConfig == false)
       */
      public String getFacetsInclude() {
        return facetsInclude;
      }

      /** Facets to include (applies when useGcsConfig == false) */
      public List setFacetsInclude(String facetsInclude) {
        this.facetsInclude = facetsInclude;
        return this;
      }

      /** Category for which to retrieve recommendations */
      @com.google.api.client.util.Key("categoryRecommendations.category")
      private String categoryRecommendationsCategory;

      /** Category for which to retrieve recommendations
       */
      public String getCategoryRecommendationsCategory() {
        return categoryRecommendationsCategory;
      }

      /** Category for which to retrieve recommendations */
      public List setCategoryRecommendationsCategory(String categoryRecommendationsCategory) {
        this.categoryRecommendationsCategory = categoryRecommendationsCategory;
        return this;
      }

      /** Whether to return extra information. */
      @com.google.api.client.util.Key("extras.enabled")
      private Boolean extrasEnabled;

      /** Whether to return extra information.
       */
      public Boolean getExtrasEnabled() {
        return extrasEnabled;
      }

      /** Whether to return extra information. */
      public List setExtrasEnabled(Boolean extrasEnabled) {
        this.extrasEnabled = extrasEnabled;
        return this;
      }

      /** Whether to return facet information */
      @com.google.api.client.util.Key("facets.enabled")
      private Boolean facetsEnabled;

      /** Whether to return facet information
       */
      public Boolean getFacetsEnabled() {
        return facetsEnabled;
      }

      /** Whether to return facet information */
      public List setFacetsEnabled(Boolean facetsEnabled) {
        this.facetsEnabled = facetsEnabled;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("relatedQueries.useGcsConfig")
      private Boolean relatedQueriesUseGcsConfig;

      /** This parameter is currently ignored
       */
      public Boolean getRelatedQueriesUseGcsConfig() {
        return relatedQueriesUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public List setRelatedQueriesUseGcsConfig(Boolean relatedQueriesUseGcsConfig) {
        this.relatedQueriesUseGcsConfig = relatedQueriesUseGcsConfig;
        return this;
      }

      /** Whether to return promotion information */
      @com.google.api.client.util.Key("promotions.enabled")
      private Boolean promotionsEnabled;

      /** Whether to return promotion information
       */
      public Boolean getPromotionsEnabled() {
        return promotionsEnabled;
      }

      /** Whether to return promotion information */
      public List setPromotionsEnabled(Boolean promotionsEnabled) {
        this.promotionsEnabled = promotionsEnabled;
        return this;
      }

      /** Channels specification */
      @com.google.api.client.util.Key
      private String channels;

      /** Channels specification
       */
      public String getChannels() {
        return channels;
      }

      /** Channels specification */
      public List setChannels(String channels) {
        this.channels = channels;
        return this;
      }

      /** Currency restriction (ISO 4217) */
      @com.google.api.client.util.Key
      private String currency;

      /** Currency restriction (ISO 4217)
       */
      public String getCurrency() {
        return currency;
      }

      /** Currency restriction (ISO 4217) */
      public List setCurrency(String currency) {
        this.currency = currency;
        return this;
      }

      /** Whether to return category recommendation information */
      @com.google.api.client.util.Key("categoryRecommendations.enabled")
      private Boolean categoryRecommendationsEnabled;

      /** Whether to return category recommendation information
       */
      public Boolean getCategoryRecommendationsEnabled() {
        return categoryRecommendationsEnabled;
      }

      /** Whether to return category recommendation information */
      public List setCategoryRecommendationsEnabled(Boolean categoryRecommendationsEnabled) {
        this.categoryRecommendationsEnabled = categoryRecommendationsEnabled;
        return this;
      }

      /** Facets to discover */
      @com.google.api.client.util.Key("facets.discover")
      private String facetsDiscover;

      /** Facets to discover
       */
      public String getFacetsDiscover() {
        return facetsDiscover;
      }

      /** Facets to discover */
      public List setFacetsDiscover(String facetsDiscover) {
        this.facetsDiscover = facetsDiscover;
        return this;
      }

      /** What extra information to return. */
      @com.google.api.client.util.Key("extras.info")
      private String extrasInfo;

      /** What extra information to return.
       */
      public String getExtrasInfo() {
        return extrasInfo;
      }

      /** What extra information to return. */
      public List setExtrasInfo(String extrasInfo) {
        this.extrasInfo = extrasInfo;
        return this;
      }

      /** Index (1-based) of first product to return */
      @com.google.api.client.util.Key
      private Long startIndex;

      /** Index (1-based) of first product to return
       */
      public Long getStartIndex() {
        return startIndex;
      }

      /** Index (1-based) of first product to return */
      public List setStartIndex(Long startIndex) {
        this.startIndex = startIndex;
        return this;
      }

      /**
     * Comma separated list of availabilities (outOfStock, limited, inStock, backOrder, preOrder,
     * onDisplayToOrder) to return
     */
      @com.google.api.client.util.Key
      private String availability;

      /** Comma separated list of availabilities (outOfStock, limited, inStock, backOrder, preOrder,
     onDisplayToOrder) to return
       */
      public String getAvailability() {
        return availability;
      }

      /**
     * Comma separated list of availabilities (outOfStock, limited, inStock, backOrder, preOrder,
     * onDisplayToOrder) to return
     */
      public List setAvailability(String availability) {
        this.availability = availability;
        return this;
      }

      /** Crowding specification */
      @com.google.api.client.util.Key
      private String crowdBy;

      /** Crowding specification
       */
      public String getCrowdBy() {
        return crowdBy;
      }

      /** Crowding specification */
      public List setCrowdBy(String crowdBy) {
        this.crowdBy = crowdBy;
        return this;
      }

      /** Search query */
      @com.google.api.client.util.Key
      private String q;

      /** Search query
       */
      public String getQ() {
        return q;
      }

      /** Search query */
      public List setQ(String q) {
        this.q = q;
        return this;
      }

      /** Whether to return spelling suggestions */
      @com.google.api.client.util.Key("spelling.enabled")
      private Boolean spellingEnabled;

      /** Whether to return spelling suggestions
       */
      public Boolean getSpellingEnabled() {
        return spellingEnabled;
      }

      /** Whether to return spelling suggestions */
      public List setSpellingEnabled(Boolean spellingEnabled) {
        this.spellingEnabled = spellingEnabled;
        return this;
      }

      /** Taxonomy name */
      @com.google.api.client.util.Key
      private String taxonomy;

      /** Taxonomy name
       */
      public String getTaxonomy() {
        return taxonomy;
      }

      /** Taxonomy name */
      public List setTaxonomy(String taxonomy) {
        this.taxonomy = taxonomy;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("spelling.useGcsConfig")
      private Boolean spellingUseGcsConfig;

      /** This parameter is currently ignored
       */
      public Boolean getSpellingUseGcsConfig() {
        return spellingUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public List setSpellingUseGcsConfig(Boolean spellingUseGcsConfig) {
        this.spellingUseGcsConfig = spellingUseGcsConfig;
        return this;
      }

      /** One of CommerceSearchUseCase, ShoppingApiUseCase */
      @com.google.api.client.util.Key
      private String useCase;

      /** One of CommerceSearchUseCase, ShoppingApiUseCase
       */
      public String getUseCase() {
        return useCase;
      }

      /** One of CommerceSearchUseCase, ShoppingApiUseCase */
      public List setUseCase(String useCase) {
        this.useCase = useCase;
        return this;
      }

      /** Location used to determine tax and shipping */
      @com.google.api.client.util.Key
      private String location;

      /** Location used to determine tax and shipping
       */
      public String getLocation() {
        return location;
      }

      /** Location used to determine tax and shipping */
      public List setLocation(String location) {
        this.location = location;
        return this;
      }

      /** Maximum number of variant results to return per result */
      @com.google.api.client.util.Key
      private Integer maxVariants;

      /** Maximum number of variant results to return per result
       */
      public Integer getMaxVariants() {
        return maxVariants;
      }

      /** Maximum number of variant results to return per result */
      public List setMaxVariants(Integer maxVariants) {
        this.maxVariants = maxVariants;
        return this;
      }

      /** Category specification */
      @com.google.api.client.util.Key("categories.include")
      private String categoriesInclude;

      /** Category specification
       */
      public String getCategoriesInclude() {
        return categoriesInclude;
      }

      /** Category specification */
      public List setCategoriesInclude(String categoriesInclude) {
        this.categoriesInclude = categoriesInclude;
        return this;
      }

      /** Boosting specification */
      @com.google.api.client.util.Key
      private String boostBy;

      /** Boosting specification
       */
      public String getBoostBy() {
        return boostBy;
      }

      /** Boosting specification */
      public List setBoostBy(String boostBy) {
        this.boostBy = boostBy;
        return this;
      }

      /** Whether safe search is enabled. Default: true */
      @com.google.api.client.util.Key
      private Boolean safe;

      /** Whether safe search is enabled. Default: true
       */
      public Boolean getSafe() {
        return safe;
      }

      /** Whether safe search is enabled. Default: true */
      public List setSafe(Boolean safe) {
        this.safe = safe;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("categories.useGcsConfig")
      private Boolean categoriesUseGcsConfig;

      /** This parameter is currently ignored
       */
      public Boolean getCategoriesUseGcsConfig() {
        return categoriesUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public List setCategoriesUseGcsConfig(Boolean categoriesUseGcsConfig) {
        this.categoriesUseGcsConfig = categoriesUseGcsConfig;
        return this;
      }

      /** Maximum number of results to return */
      @com.google.api.client.util.Key
      private Long maxResults;

      /** Maximum number of results to return
       */
      public Long getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return */
      public List setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Whether to return facet information as configured in the GCS account */
      @com.google.api.client.util.Key("facets.useGcsConfig")
      private Boolean facetsUseGcsConfig;

      /** Whether to return facet information as configured in the GCS account
       */
      public Boolean getFacetsUseGcsConfig() {
        return facetsUseGcsConfig;
      }

      /** Whether to return facet information as configured in the GCS account */
      public List setFacetsUseGcsConfig(Boolean facetsUseGcsConfig) {
        this.facetsUseGcsConfig = facetsUseGcsConfig;
        return this;
      }

      /** Whether to return category information */
      @com.google.api.client.util.Key("categories.enabled")
      private Boolean categoriesEnabled;

      /** Whether to return category information
       */
      public Boolean getCategoriesEnabled() {
        return categoriesEnabled;
      }

      /** Whether to return category information */
      public List setCategoriesEnabled(Boolean categoriesEnabled) {
        this.categoriesEnabled = categoriesEnabled;
        return this;
      }

      /** Comma separated list of attributes to return */
      @com.google.api.client.util.Key
      private String attributeFilter;

      /** Comma separated list of attributes to return
       */
      public String getAttributeFilter() {
        return attributeFilter;
      }

      /** Comma separated list of attributes to return */
      public List setAttributeFilter(String attributeFilter) {
        this.attributeFilter = attributeFilter;
        return this;
      }

      /** Whether to add a click tracking parameter to offer URLs */
      @com.google.api.client.util.Key
      private Boolean clickTracking;

      /** Whether to add a click tracking parameter to offer URLs
       */
      public Boolean getClickTracking() {
        return clickTracking;
      }

      /** Whether to add a click tracking parameter to offer URLs */
      public List setClickTracking(Boolean clickTracking) {
        this.clickTracking = clickTracking;
        return this;
      }

      /** Image thumbnails specification */
      @com.google.api.client.util.Key
      private String thumbnails;

      /** Image thumbnails specification
       */
      public String getThumbnails() {
        return thumbnails;
      }

      /** Image thumbnails specification */
      public List setThumbnails(String thumbnails) {
        this.thumbnails = thumbnails;
        return this;
      }

      /** Language restriction (BCP 47) */
      @com.google.api.client.util.Key
      private String language;

      /** Language restriction (BCP 47)
       */
      public String getLanguage() {
        return language;
      }

      /** Language restriction (BCP 47) */
      public List setLanguage(String language) {
        this.language = language;
        return this;
      }

      /** Category recommendation specification */
      @com.google.api.client.util.Key("categoryRecommendations.include")
      private String categoryRecommendationsInclude;

      /** Category recommendation specification
       */
      public String getCategoryRecommendationsInclude() {
        return categoryRecommendationsInclude;
      }

      /** Category recommendation specification */
      public List setCategoryRecommendationsInclude(String categoryRecommendationsInclude) {
        this.categoryRecommendationsInclude = categoryRecommendationsInclude;
        return this;
      }

      /** Country restriction (ISO 3166) */
      @com.google.api.client.util.Key
      private String country;

      /** Country restriction (ISO 3166)
       */
      public String getCountry() {
        return country;
      }

      /** Country restriction (ISO 3166) */
      public List setCountry(String country) {
        this.country = country;
        return this;
      }

      /** Ranking specification */
      @com.google.api.client.util.Key
      private String rankBy;

      /** Ranking specification
       */
      public String getRankBy() {
        return rankBy;
      }

      /** Ranking specification */
      public List setRankBy(String rankBy) {
        this.rankBy = rankBy;
        return this;
      }

      /** Restriction specification */
      @com.google.api.client.util.Key
      private String restrictBy;

      /** Restriction specification
       */
      public String getRestrictBy() {
        return restrictBy;
      }

      /** Restriction specification */
      public List setRestrictBy(String restrictBy) {
        this.restrictBy = restrictBy;
        return this;
      }

      /** Return empty facet buckets. */
      @com.google.api.client.util.Key("facets.includeEmptyBuckets")
      private Boolean facetsIncludeEmptyBuckets;

      /** Return empty facet buckets.
       */
      public Boolean getFacetsIncludeEmptyBuckets() {
        return facetsIncludeEmptyBuckets;
      }

      /** Return empty facet buckets. */
      public List setFacetsIncludeEmptyBuckets(Boolean facetsIncludeEmptyBuckets) {
        this.facetsIncludeEmptyBuckets = facetsIncludeEmptyBuckets;
        return this;
      }

      /** Whether to return redirect information */
      @com.google.api.client.util.Key("redirects.enabled")
      private Boolean redirectsEnabled;

      /** Whether to return redirect information
       */
      public Boolean getRedirectsEnabled() {
        return redirectsEnabled;
      }

      /** Whether to return redirect information */
      public List setRedirectsEnabled(Boolean redirectsEnabled) {
        this.redirectsEnabled = redirectsEnabled;
        return this;
      }

      /** Whether to return redirect information as configured in the GCS account */
      @com.google.api.client.util.Key("redirects.useGcsConfig")
      private Boolean redirectsUseGcsConfig;

      /** Whether to return redirect information as configured in the GCS account
       */
      public Boolean getRedirectsUseGcsConfig() {
        return redirectsUseGcsConfig;
      }

      /** Whether to return redirect information as configured in the GCS account */
      public List setRedirectsUseGcsConfig(Boolean redirectsUseGcsConfig) {
        this.redirectsUseGcsConfig = redirectsUseGcsConfig;
        return this;
      }

      /** Whether to return related queries */
      @com.google.api.client.util.Key("relatedQueries.enabled")
      private Boolean relatedQueriesEnabled;

      /** Whether to return related queries
       */
      public Boolean getRelatedQueriesEnabled() {
        return relatedQueriesEnabled;
      }

      /** Whether to return related queries */
      public List setRelatedQueriesEnabled(Boolean relatedQueriesEnabled) {
        this.relatedQueriesEnabled = relatedQueriesEnabled;
        return this;
      }

      /** This parameter is currently ignored */
      @com.google.api.client.util.Key("categoryRecommendations.useGcsConfig")
      private Boolean categoryRecommendationsUseGcsConfig;

      /** This parameter is currently ignored
       */
      public Boolean getCategoryRecommendationsUseGcsConfig() {
        return categoryRecommendationsUseGcsConfig;
      }

      /** This parameter is currently ignored */
      public List setCategoryRecommendationsUseGcsConfig(Boolean categoryRecommendationsUseGcsConfig) {
        this.categoryRecommendationsUseGcsConfig = categoryRecommendationsUseGcsConfig;
        return this;
      }

      /** Whether to return promotion information as configured in the GCS account */
      @com.google.api.client.util.Key("promotions.useGcsConfig")
      private Boolean promotionsUseGcsConfig;

      /** Whether to return promotion information as configured in the GCS account
       */
      public Boolean getPromotionsUseGcsConfig() {
        return promotionsUseGcsConfig;
      }

      /** Whether to return promotion information as configured in the GCS account */
      public List setPromotionsUseGcsConfig(Boolean promotionsUseGcsConfig) {
        this.promotionsUseGcsConfig = promotionsUseGcsConfig;
        return this;
      }

    }

  }

  /**
   * Builder for {@link Shopping}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Shopping}. */
    @Override
    public Shopping build() {
      return new Shopping(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link ShoppingRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setShoppingRequestInitializer(
        ShoppingRequestInitializer shoppingRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(shoppingRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
